// -*- C++ -*-
/*!
 * @file ConfigAdmin.h
 * @brief Configuration Administration classes
 * @date $Date: 2007-12-31 03:08:02 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2007-2008
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ConfigAdmin.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef ConfigAdmin_h
#define ConfigAdmin_h

#include <rtm/Properties.h>
#include <rtm/StringUtil.h>
#include <string>
#include <vector>
#include <iostream>

/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RTݡͥ
 *
 * @else
 *
 * @namespace RTC
 *
 * @brief RT-Component
 *
 * @endif
 */
namespace RTC
{
  //============================================================
  // ConfigBase class
  //============================================================
  /*!
   * @if jp
   * @class ConfigBase
   * @brief ConfigBase ݥ饹
   * 
   * Ƽ拾ե졼ݻ뤿ݥ饹
   * ݥե졼󥯥饹ϡʲν貾۴ؿμ󶡤ʤ
   * Фʤʤ
   *
   * public󥿡եȤưʲΤΤ󶡤롣
   * - update(): ե졼ѥ᡼ͤι
   *
   * @since 0.4.0
   *
   * @else
   * @class ConfigBase
   * @brief ConfigBase abstract class
   *
   * This is the abstract interface class to hold various configuration 
   * information.
   * Concrete configuration classes must implement the following pure virtual
   * functions.
   *
   * This class provides public interface as follows.
   * - update(): update configuration parameter value
   *
   * @since 0.4.0
   *
   * @endif
   */
  class ConfigBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief 󥹥ȥ饯
     * 
     * 󥹥ȥ饯
     *
     * @param name_ ե졼̾
     * @param def_val ʸΥǥե
     * 
     * @else
     *
     * @brief Constructer
     *
     * Constructer
     *
     * @param name_ Configuration name
     * @param def_val Default value in string format
     *
     * @endif
     */
    ConfigBase(const char* name_, const char* def_val)
      : name(name_), default_value(def_val) {}
    
    /*!
     * @if jp
     *
     * @brief ۥǥȥ饯
     * 
     * ۥǥȥ饯
     *
     * @else
     *
     * @brief Virtual Destructor
     *
     * Virtual Destructor
     *
     * @endif
     */
    virtual ~ConfigBase(){};
    
    /*!
     * @if jp
     *
     * @brief ե졼ѥ᡼͹ѽ貾۴ؿ
     * 
     * ե졼ͤǥե졼ѥ᡼򹹿
     * ν貾۴ؿ
     *
     * @param val ѥ᡼ͤʸɽ
     *
     * @return 
     * 
     * @else
     *
     * @brief Pure virtual function to update configuration parameter values
     * 
     * Pure virtual function to update configuration parameter 
     * by the configuration value.
     *
     * @param val The parameter values converted into character string format
     *
     * @return Result of the setup
     *
     * @endif
     */
    virtual bool update(const char* val) = 0;
    
    /*!
     * @if jp
     * @brief  ե졼̾
     * @else
     * @brief  Configuration name
     * @endif
     */
    const char* name;
    
    /*!
     * @if jp
     * @brief  ʸΥǥե
     * @else
     * @brief  Default value in string format
     * @endif
     */
    const char* default_value;
  };
  
  //============================================================
  // Config template class
  //============================================================
  /*!
   * @if jp
   * @class Config
   * @brief Config 饹
   * 
   * ե졼ѥ᡼ξݻ륯饹
   * \<VarType\>Ȥƥե졼Υǡꤹ롣
   * \<TransFunc\>Ȥꤵ줿ǡʸѴѴؿꤹ롣
   *
   * @param VarType ե졼ѥ᡼Ǽѿ
   * @param TransFunc ǼǡʸѴѴؿ
   *
   * @since 0.4.0
   *
   * @else
   * @class Config
   * @brief Config class
   * 
   * Class to hold the configuration parameter information.
   * Specify the data type of the configuration as \<VarType\>
   * Specify transformation function to convert data type set as \<TransFunc\>
   * into string format.
   *
   * @param VarType Cariable to hold configuration parameter
   * @param TransFunc Transformation function to transform the stored data 
   * type into string format.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <typename VarType,
	    typename TransFunc = bool (*)(VarType&, const char*)>
  class Config
    : public ConfigBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief 󥹥ȥ饯
     * 
     * 󥹥ȥ饯
     *
     * @param name ե졼ѥ᡼̾
     * @param var ե졼ѥ᡼Ǽѿ
     * @param def_val ʸΥǥե
     * @param trans ʸѴؿ
     * 
     * @else
     *
     * @brief Constructor
     * 
     * Constructor
     *
     * @param name Configuration parameter name
     * @param var Configuration parameter variable
     * @param def_val Default value in string format
     * @param trans Function to transform into string format
     *
     * @endif
     */
    Config(const char* name, VarType& var, const char* def_val,
	   TransFunc trans = stringTo)
      : ConfigBase(name, def_val), m_var(var), m_trans(trans)
    {
    }
    
    /*!
     * @if jp
     *
     * @brief ۥǥȥ饯
     * 
     * ۥǥȥ饯
     *
     * @else
     *
     * @brief Virtual Destructor
     * 
     * Virtual Destructor.
     *
     * @endif
     */
    virtual ~Config(){}
    
    /*!
     * @if jp
     *
     * @brief Хɥѥ᡼ͤ򹹿
     * 
     * ե졼ͤǥե졼ѥ᡼򹹿
     *
     * @param val ѥ᡼ͤʸɽ
     *
     * @return (:true:false)
     * 
     * @else
     *
     * @brief Update a bind parameter value
     * 
     * Update configuration paramater by the configuration value.
     *
     * @param val The parameter values converted into character string format
     *
     * @return Update result (Successful:true, Failed:false)
     *
     * @endif
     */
    virtual bool update(const char* val)
    {
      if ((*m_trans)(m_var, val)) return true;
      (*m_trans)(m_var, default_value);
      return false;
    }
    
  protected:
    /*!
     * @if jp
     * @brief  ե졼ѥ᡼Ǽѿ
     * @else
     * @brief  Configuration parameter variable
     * @endif
     */
    VarType& m_var;
    
    /*!
     * @if jp
     * @brief  ե졼ѥ᡼ʸѴؿ
     * @else
     * @brief  Transformation function to convert configuration parameter type 
     *         into string format.
     * @endif
     */
    TransFunc m_trans;
  };
  
  //============================================================
  // ConfigAdmin class
  //============================================================
  /*!
   * @if jp
   * @class ConfigAdmin
   * @brief ConfigAdmin 饹
   * 
   * Ƽ拾ե졼륯饹
   *
   * @since 0.4.0
   *
   * @else
   * @class ConfigAdmin
   * @brief ConfigAdmin class
   * 
   * Class to manage various configuration information.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class ConfigAdmin
  {
  public:
    /*!
     * @if jp
     *
     * @brief 󥹥ȥ饯
     * 
     * 󥹥ȥ饯
     *
     * @param prop оݥץѥƥ̾
     * 
     * @else
     *
     * @brief Constructor
     * 
     * Constructor
     *
     * @param prop The target property name for setup
     *
     * @endif
     */
    ConfigAdmin(RTC::Properties& prop);
    
    /*!
     * @if jp
     *
     * @brief ۥǥȥ饯
     * 
     * ۥǥȥ饯
     *
     * @else
     *
     * @brief Virtual Destructor
     * 
     * Virtual Destructor
     *
     * @endif
     */
    ~ConfigAdmin();
    
    /*!
     * @if jp
     *
     * @brief ե졼ѥ᡼
     * 
     * ե졼ѥ᡼ѿХɤ
     * ꤷ̾ΤΥե졼ѥ᡼¸ߤ
     * false֤
     * \<VarType\>Ȥƥե졼ѥ᡼Υǡꤹ롣
     *
     * @param param_name ե졼ѥ᡼̾
     * @param var ե졼ѥ᡼Ǽѿ
     * @param def_val ե졼ѥ᡼ǥե
     * @param trans ե졼ѥ᡼ʸѴѴؿ
     *
     * @return (:true꼺:false)
     * 
     * @else
     *
     * @brief Setup for configuration parameters
     * 
     * Bind configuration parameter to its variable.
     * Return false, if configuration parameter of specified name has already 
     * existed.
     * Specify the data type of the configuration as \<VarType\>.
     *
     * @param param_name Configuration parameter name
     * @param var Configuration parameter variable
     * @param def_val Default value of configuration parameter
     * @param trans Function to transform configuration parameter type into 
     *        string format
     *
     * @return Setup result (Successful:true, Failed:false)
     *
     * @endif
     */
    template <typename VarType>
    bool bindParameter(const char* param_name, VarType& var,
		       const char* def_val,
		       bool (*trans)(VarType&, const char*) = ::stringTo)
    {
      if (isExist(param_name)) return false;
      if (!::stringTo(var, def_val)) return false;
      m_params.push_back(new Config<VarType>(param_name, var, def_val, trans));
      return true;
    }
    
    /*!
     * @if jp
     *
     * @brief ե졼ѥ᡼ι(ID)
     * 
     * ꤷIDΥե졼󥻥åȤꤷͤǡ
     * ե졼ѥ᡼ͤ򹹿
     * ꤷIDΥե졼󥻥åȤ¸ߤʤϡ
     * ⤻˽λ롣
     *
     * @param config_set оݤΥե졼󥻥åID
     * 
     * @else
     *
     * @brief Update configuration parameter (By ID)
     * 
     * Update comfiguration parameter value by the value that 
     * set to a configuration set of specified ID.
     * Exit without doing anthing if a configuration set of specified ID 
     * does not exist.
     *
     * @param config_set The target configuration set's ID to setup
     *
     * @endif
     */
    void update(const char* config_set);
    
    /*!
     * @if jp
     *
     * @brief ե졼ѥ᡼ι(̾λ)
     * 
     * ꤷѥΥե졼ꤷͤǡ
     * ե졼ѥ᡼ͤ򹹿
     *
     * @param config_set ե졼̾Ρ.׶ڤǺǸǤ
     *                   ̾
     * @param config_param ե졼󥻥åȤκǸ̾
     * 
     * @else
     *
     * @brief Update the values of configuration parameters (By name)
     * 
     * Update the configuration value by the value that set to a configuration
     * set value of specified name.
     *
     * @param config_set configuration name. Name that each separates 
     *        by each comma(.) and excludes the last element.
     * @param config_param Last element name of configuration set
     *
     * @endif
     */
    void update(const char* config_set, const char* config_param);
    
    /*!
     * @if jp
     *
     * @brief ե졼ѥ᡼ι
     *        (ƥ֥ե졼󥻥å)
     * 
     * ե졼󥻥åȤƤˡ
     * ߥƥ֤ˤʤäƤ륳ե졼ꤷͤǡ
     * ե졼ѥ᡼ͤ򹹿롣
     * νǤιϡƥ֤ȤʤäƤ륳ե졼󥻥åȤ
     * ¸ߤƤ硢ι饳ե졼󥻥åȤƤ
     * ƤΤ߼¹Ԥ롣
     *
     * @else
     *
     * @brief Update the values of configuration parameters
     *        (Active configuration set)
     * 
     * When configuration set is updated, update the configuration parameter 
     * value to the value that is set to the current active configuration.
     * This update will be executed, only when an active configuration set 
     * exists and the content of the configuration set has been updated from 
     * the last update.
     *
     * @endif
     */
    void update();
    
    /*!
     * @if jp
     *
     * @brief ե졼ѥ᡼¸߳ǧ
     * 
     * ꤷ̾Τĥե졼ѥ᡼¸ߤ뤫ǧ롣
     *
     * @param name ե졼ѥ᡼̾Ρ
     *
     * @return ¸߳ǧ(ѥ᡼:trueѥ᡼ʤ:false)
     *
     * @else
     *
     * @brief Check the existence of configuration parameters
     * 
     * Check the existence of configuration parameters of specified name.
     *
     * @param name Configuration parameter name
     *
     * @return Result of existance confirmation 
     *         (Parameters exist:true, else:false)
     *
     * @endif
     */
    bool isExist(const char* name);
    
    /*!
     * @if jp
     *
     * @brief ե졼ѥ᡼ѹǧ
     * 
     * ե졼ѥ᡼ѹ줿ǧ롣
     *
     * @return ѹǧ(ѹ:trueѹʤ:false)
     *
     * @else
     *
     * @brief Confirm to change configuration parameters
     * 
     * Confirm that configuration parameters have changed.
     *
     * @return Result of change confirmation
     *         (There is a change:trueNo change:false)
     *
     * @endif
     */
    bool isChanged() {return m_changed;}
    
    /*!
     * @if jp
     *
     * @brief ƥ֡ե졼󥻥åIDμ
     * 
     * ߥƥ֤ʥե졼󥻥åȤID롣
     *
     * @return ƥ֡ե졼󥻥åID
     *
     * @else
     *
     * @brief Get ID of active configuration set
     * 
     * Get ID of the current active configuration set.
     *
     * @return The active configuration set ID
     *
     * @endif
     */
    const char* getActiveId() {return m_activeId.c_str();}
    
    /*!
     * @if jp
     *
     * @brief ե졼󥻥åȤ¸߳ǧ
     * 
     * ꤷե졼󥻥åȤ¸ߤ뤫ǧ롣
     *
     * @param config_id ǧоݥե졼󥻥åID
     *
     * @return ¸߳ǧ(ꤷConfigSet:trueʤ:false)
     *
     * @else
     *
     * @brief Check the existence of configuration set
     * 
     * Check the existence of specified configuration set.
     *
     * @param config_id ID of target configuration set for confirmation
     *
     * @return Result of existence confirmation 
     *         (Specified ConfigSet exists:true, else:false)
     *
     * @endif
     */
    bool haveConfig(const char* config_id)
    {
      return (m_configsets.hasKey(config_id) == NULL) ? false : true;
    }
    
    /*!
     * @if jp
     *
     * @brief ե졼󥻥åȤΥƥֲǧ
     * 
     * ե졼󥻥åȤƥֲƤ뤫ǧ롣
     *
     * @return ֳǧ(ƥ־:true󥢥ƥ־:false)
     *
     * @else
     *
     * @brief Confirm to activate configuration set
     * 
     * Confirm that configuration set has been activated.
     *
     * @return Result of state confirmation
     *         (Active state:true, Inactive state:false)
     *
     * @endif
     */
    bool isActive()
    {
      return m_active;
    }
    //    const std::vector<Properties*>* getConfigurationParameterValues();
    //    const Properties* getConfigurationParameterValue(const char* name);
    //    bool setConfigurationParameter(const char* name, const char* value);
    
    /*!
     * @if jp
     *
     * @brief ե졼󥻥åȤμ
     * 
     * ꤵƤե졼󥻥åȤ롣
     *
     * @return ե졼󥻥å
     *
     * @else
     *
     * @brief Get all configuration sets
     * 
     * Get all specified configuration sets
     *
     * @return All configuration sets
     *
     * @endif
     */
    const std::vector<Properties*>& getConfigurationSets();
    
    /*!
     * @if jp
     *
     * @brief ꤷIDΥե졼󥻥åȤμ
     * 
     * IDǻꤷե졼󥻥åȤ롣
     * ꤷե졼󥻥åȤ¸ߤʤϡ
     * Υե졼󥻥åȤ֤
     *
     * @param config_id оݥե졼󥻥åȤID
     *
     * @return ե졼󥻥å
     *
     * @else
     *
     * @brief Get a configuration set by specified ID
     * 
     * Get a configuration set that was specified by ID
     * Return empty configuration set, if a configuration set of
     * specified ID doesn't exist.
     *
     * @param config_id ID of the target configuration set for getting
     *
     * @return The configuration set
     *
     * @endif
     */
    const Properties& getConfigurationSet(const char* config_id);
    
    /*!
     * @if jp
     *
     * @brief ꤷץѥƥΥե졼󥻥åȤؤɲ
     * 
     * ꤷץѥƥIDǻꤷե졼󥻥åȤɲä롣
     * ꤷIDȰפ륳ե졼󥻥åȤ¸ߤʤϡ
     * false ֤
     *
     * @param config_id ɲоݥե졼󥻥åȤID
     * @param configuration_set ɲäץѥƥ
     *
     * @return ɲý¹Է(ɲ:trueɲü:false)
     *
     * @else
     *
     * @brief Add to configuration set from specified property
     * 
     * Add specified property to configuration set that was specified by ID.
     * Return false if configuration set, that matches specified ID, 
     * doesn't exist.
     *
     * @param config_id ID of the target configuration set for add
     * @param configuration_set Property to add
     *
     * @return Add result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool setConfigurationSetValues(const char* config_id,
				   const RTC::Properties& configuration_set);
    
    /*!
     * @if jp
     *
     * @brief ƥ֡ե졼󥻥åȤ
     * 
     * ߥƥ֤ȤʤäƤ륳ե졼󥻥åȤ롣
     * ƥ֤ȤʤäƤ륳ե졼󥻥åȤ¸ߤʤϡ
     * Υե졼󥻥å ֤
     *
     * @return ƥ֡ե졼󥻥å
     *
     * @else
     *
     * @brief Get the active configuration set
     * 
     * Get the current active configuration set.
     * Return empty configuration set, if an active configuration set 
     * doesn't exist.
     *
     * @return The active configuration set
     *
     * @endif
     */
    const Properties& getActiveConfigurationSet();
    
    /*!
     * @if jp
     *
     * @brief ե졼󥻥åȤͤɲ
     * 
     * ե졼󥻥åȤͤɲä롣
     *
     * @param configuration_set ɲäץѥƥ
     *
     * @return ɲý(ɲ:trueɲü:false)
     *
     * @else
     *
     * @brief Add the configuration value to configuration set
     * 
     * Add the configuration value to configuration set
     *
     * @param configuration_set Property to add
     *
     * @return Add Result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool addConfigurationSet(const Properties& configuration_set);
    
    /*!
     * @if jp
     *
     * @brief ե졼󥻥åȤκ
     * 
     * ꤷIDΥե졼󥻥åȤ롣
     * ꤷIDΥե졼󥻥åȤ¸ߤʤϡ
     * false֤
     *
     * @param config_id оݥե졼󥻥åȤID
     *
     * @return (:true:false)
     *
     * @else
     *
     * @brief Remove the configuration set
     * 
     * Remove the configuration set of specified ID
     * Return empty configuration set, if a configuration set of
     * specified ID doesn't exist.
     *
     * @param config_id ID of the target configuration set for remove
     *
     * @return Remove result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool removeConfigurationSet(const char* config_id);
    
    /*!
     * @if jp
     *
     * @brief ե졼󥻥åȤΥƥֲ
     * 
     * ꤷIDΥե졼󥻥åȤ򥢥ƥֲ롣
     * ꤷIDΥե졼󥻥åȤ¸ߤʤϡ
     * false֤
     *
     * @param config_id оݥե졼󥻥åȤID
     *
     * @return ƥֽ(:true:false)
     *
     * @else
     *
     * @brief Activate the configuration set
     * 
     * Activate the configuration set of specified ID
     * Return empty configuration set, if a configuration set of
     * specified ID doesn't exist.
     *
     * @param config_id ID of the target configuration set for remove
     *
     * @return Activate result (Remove success:trueRemove failure:false)
     *
     * @endif
     */
    bool activateConfigurationSet(const char* config_id);
    
  private:
    ConfigAdmin(const ConfigAdmin& ca) : m_configsets(ca.m_configsets) {};
    ConfigAdmin& operator=(const ConfigAdmin& ca){return *this;};
    
    struct find_conf
    {
      std::string m_name;
      find_conf(const char* name) : m_name(name) {};
      bool operator()(ConfigBase* conf)
      {
	return m_name == conf->name;
      }
    };
    
    RTC::Properties& m_configsets;
    RTC::Properties  m_emptyconf;
    std::vector<ConfigBase*> m_params;
    std::string m_activeId;
    bool m_active;
    bool m_changed;
    std::vector<std::string> m_newConfig;
  };
}; // namespace RTC
#endif // ConfigAdmin_h
