// -*- C++ -*-
/*!
 * @file DataInPort.cpp
 * @brief Base class of InPort
 * @date $Date: 2007-12-31 03:08:02 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: DataInPort.cpp 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#include <rtm/DataInPort.h>
#include <rtm/CORBA_SeqUtil.h>
#include <rtm/NVUtil.h>
#include <iostream>
#include <algorithm>

namespace RTC
{
  /*!
   * @if jp
   * @brief InterfaceFunctor
   * @else
   * @brief Functor to publish the interface
   * @endif
   */
  struct DataInPort::publish
  {
    publish(SDOPackage::NVList& prop) : m_prop(prop) {}
    void operator()(InPortProvider* provider)
    {
      provider->publishInterface(m_prop);
    }
    SDOPackage::NVList& m_prop;
  };
  
  /*!
   * @if jp
   * @brief Interface³Functor
   * @else
   * @brief Functor to subscribe the interface
   * @endif
   */
  struct DataInPort::subscribe
  {
    subscribe(const SDOPackage::NVList& prop) : m_prop(prop) {}
    void operator()(OutPortConsumer* consumer)
    {
      consumer->subscribeInterface(m_prop);
    }
    const SDOPackage::NVList& m_prop;
  };
  
  /*!
   * @if jp
   * @brief Interface³Functor
   * @else
   * @brief Functor to unsubscribe the interface
   * @endif
   */
  struct DataInPort::unsubscribe
  {
    unsubscribe(const SDOPackage::NVList& prop) : m_prop(prop) {}
    void operator()(OutPortConsumer* consumer)
    {
      consumer->unsubscribeInterface(m_prop);
    }
    const SDOPackage::NVList& m_prop;
  };
  
  /*!
   * @if jp
   * @brief ǥȥ饯
   * @else
   * @brief Destructor
   * @endif
   */
  DataInPort::~DataInPort()
  {
  }
  
  //============================================================
  // protected interfaces
  //============================================================
  
  /*!
   * @if jp
   * @brief Interface
   * @else
   * @brief Publish interface information
   * @endif
   */
  ReturnCode_t
  DataInPort::publishInterfaces(ConnectorProfile& connector_profile)
  {    
    if (m_dummy.size() != 1)
      {
	std::cerr << "Memory access violation was detected." << std::endl;
	std::cerr << "dummy.size(): " << m_dummy.size() << std::endl;
	std::cerr << "size() should be 1." << std::endl;
      }
    std::for_each(m_providers.begin(), m_providers.end(),
		  publish(connector_profile.properties));
    
    return RTC::RTC_OK;
  }
  
  /*!
   * @if jp
   * @brief Interface³
   * @else
   * @brief Subscribe to the interface
   * @endif
   */
  ReturnCode_t
  DataInPort::subscribeInterfaces(const ConnectorProfile& connector_profile)
  {
    std::for_each(m_consumers.begin(), m_consumers.end(),
		  subscribe(connector_profile.properties));
    return RTC::RTC_OK;
  }
  
  /*!
   * @if jp
   * @brief Interfaceؤ³
   * @else
   * @brief Disconnect the interface connection
   * @endif
   */
  void
  DataInPort::unsubscribeInterfaces(const ConnectorProfile& connector_profile)
  {
    std::for_each(m_consumers.begin(), m_consumers.end(),
		  unsubscribe(connector_profile.properties));
  }
  
};
