// -*- C++ -*-
/*!
 * @file DataInPort.h
 * @brief RTC::Port implementation for Data InPort
 * @date $Date: 2008-01-13 15:06:40 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: DataInPort.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef DataInPort_h
#define DataInPort_h

#include <rtm/PortBase.h>
#include <rtm/BufferBase.h>
#include <rtm/InPortCorbaProvider.h>
#include <rtm/OutPortCorbaConsumer.h>
#include <rtm/NVUtil.h>
#include <rtm/InPort.h>

#ifdef RTC_SOCK_DATAPORT
#include <rtm/InPortTcpSockProvider.h>
#endif

/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RTݡͥ
 *
 * @else
 *
 * @namespace RTC
 *
 * @brief RT-Component
 *
 * @endif
 */
namespace RTC
{
  /*!
   * @if jp
   * @class DataInPort
   * @brief InPort  Port
   *
   * ǡϥݡȤμ饹
   *
   * @since 0.4.0
   *
   * @else
   * @class DataInPort
   * @brief Port for InPort
   *
   * This is an implementation class for the data input port.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class DataInPort
    : public virtual PortBase
  {
  public:
    /*!
     * @if jp
     * @brief 󥹥ȥ饯
     *
     * 󥹥ȥ饯
     *
     * @param name ݡ̾
     * @param inport ǡϥݡȤ˴ϢդInPort֥
     *               InPort֥ȤǰǡХåեפꤹ
     * @param prop ݡѥץѥƥ
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @param name Port name
     * @param inport InPort object that is associated with this data input port.
     *               Specify also the data type and the buffer type used in 
     *               the InPort object.
     * @param prop Property for setting ports
     *
     * @endif
     */
    template <class DataType, template <class DataType> class Buffer>
    DataInPort(const char* name, InPort<DataType, Buffer>& inport,
	       Properties& prop)
      : PortBase(name)
    {
      // PortProfile::properties 
      addProperty("port.port_type", "DataInPort");
      
      // CORBA InPort Provider
      m_providers.push_back(new InPortCorbaProvider<DataType>(inport));
      m_providers.back()->publishInterfaceProfile(m_profile.properties);
      
      // TCP Socket InPort Provider
#ifdef RTC_SOCK_DATAPORT
      m_providers.push_back(new InPortTcpSockProvider<DataType>(inport, prop));
      m_providers.back()->publishInterfaceProfile(m_profile.properties);
#endif
      
      // CORBA OutPort Consumer
      m_consumers.push_back(new OutPortCorbaConsumer<DataType>(inport));
      
      // dummy var
      m_dummy.push_back(1);
    }
    
    /*!
     * @if jp
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~DataInPort();
    
  protected:
    /*!
     * @if jp
     * @brief Interface
     *
     * Interface롣
     * PortͭƤץХ(Provider)˴ؤ
     * ConnectorProfile#properties롣
     *
     * @param connector_profile ͥץե
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     * @brief Publish interface information
     *
     * Publish interface information.
     * Assign the Provider information that owned by this port
     * to ConnectorProfile#properties
     *
     * @param connector_profile The connector profile
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t
    publishInterfaces(ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     * @brief Interface³
     *
     * Interface³롣
     * PortͭConsumerŬ礹Provider˴ؤ 
     * ConnectorProfile#properties Ф
     * ConsumerCORBA֥ȻȤꤹ롣
     *
     * @param connector_profile ͥץե
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     * @brief Subscribe to the interface
     *
     * Subscribe to interface.
     * Derive Provider information that matches Consumer owned by the Port 
     * from ConnectorProfile#properties and 
     * set the Consumer to the reference of the CORBA object.
     *
     * @param connector_profile The connector profile
     *
     * @return ReturnCode_t The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t
    subscribeInterfaces(const ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     * @brief Interfaceؤ³
     *
     * Interfaceؤ³롣
     * Ϳ줿ConnectorProfile˴ϢConsumerꤵ줿ƤObject
     * ³롣
     *
     * @param connector_profile ͥץե
     *
     * @else
     * @brief Disconnect the interface connection
     *
     * Disconnect the interface connection.
     * Release all objects set in Consumer associated with 
     * given ConnectorProfile and unscribe the interface.
     *
     * @param connector_profile The connector profile
     *
     * @endif
     */
    virtual void
    unsubscribeInterfaces(const ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     * @brief InterfaceFunctor
     *
     * Interface¹Ԥ뤿Functor
     *
     * @else
     * @brief Functor to publish the interface
     *
     * Functor to publish the interface
     *
     * @endif
     */
    struct publish;
    
    /*!
     * @if jp
     * @brief Interface³Functor
     *
     * Interface³¹Ԥ뤿Functor
     *
     * @else
     * @brief Functor to subscribe the interface
     *
     * Functor to subscribe the interface
     *
     * @endif
     */
    struct subscribe;
    
    /*!
     * @if jp
     * @brief Interface³Functor
     *
     * Interface³¹Ԥ뤿Functor
     *
     * @else
     * @brief Functor to unsubscribe the interface
     *
     * Functor to unsubscribe the interface
     *
     * @endif
     */
    struct unsubscribe;
    
  private:
    /*
     * Bug
     *
     * Fedora5/gcc4.1.1. ˤ DataInPort ƬΰΥ˲
     * äƤ͡
     * (gdbǤʤǴäɤäƤߤǰ)
     * ȤȤ std::vector<InPortProvider*> m_providers Ƭ
     * Τޤޤȡ
     * $vec->_M_impl._M_start:  begin() 
     * $vec->_M_impl._M_finish: end() 
     * ˤƤ _M_start ԲĲʰ֤ؤᡢ
     * m_providers Ф for_each ŬѤSegv롣
     *
     * FreeBSD6.0/gcc3.4.4ǤϤθݤȯ
     */
    std::vector<int> m_dummy;
    std::vector<InPortProvider*> m_providers;
    std::vector<OutPortConsumer*> m_consumers;
  };
}; // namespace RTC

#endif // DataInPort_h
