// -*- C++ -*-
/*!
 * @file  InPortConsumer.h
 * @brief InPortConsumer class
 * @date  $Date: 2007-12-31 03:08:03 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPortConsumer.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef InPortConsumer_h
#define InPortConsumer_h

#include <rtm/NVUtil.h>
#include <rtm/CorbaConsumer.h>

namespace RTC
{
  /*!
   * @if jp
   *
   * @class InPortConsumer
   *
   * @brief InPortConsumer ݥ饹
   *
   * ϥݡȥ󥷥塼ޤΤݥ󥿡ե饹
   * ƶݥ饹ϡʲν貾۴ؿμ󶡤ʤФʤʤ
   * - push(): ǡ
   * - clone(): ݡȤΥԡ
   * - subscribeInterface(): ǡΤؤϿ
   * - unsubscribeInterface(): ǡΤϿ
   *
   * @since 0.4.0
   *
   * @else
   * @class InPortConsumer
   *
   * @brief InPortConsumer abstract class
   *
   * This is the abstract interface class for the input port Consumer.
   * Concrete classes must implement the following pure virtual functions.
   * - push(): Send data
   * - clone(): Copy ports
   * - subscribeInterface(): Subscribe the data send notification
   * - unsubscribeInterface(): Unsubscribe the data send notification
   *
   * @since 0.4.0
   *
   * @endif
   *
   */
  class InPortConsumer
  {
  public:
    /*!
     * @if jp
     * @brief 󥹥ȥ饯
     *
     * 󥹥ȥ饯
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @endif
     */
    //InPortConsumer();
    
    /*!
     * @if jp
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~InPortConsumer(){};
    
    /*!
     * @if jp
     * @brief ³ؤΥǡ
     *
     * ³ΥݡȤإǡ뤿ν貾۴ؿ
     *
     * @else
     * @brief Send data to the destination port
     *
     * Pure virtual function to send data to the destination port.
     *
     * @endif
     */
    virtual void push() = 0;
    
    /*!
     * @if jp
     * @brief ݡȤΥԡ
     *
     * ݡȤΥԡ뤿ν貾۴ؿ
     *
     * @return ʣ줿 InPortConsumer ֥
     *
     * @else
     * @brief Clone this port
     *
     * Pure virtual function to generate this clone port.
     *
     * @return The clone InPortConsumer object
     *
     * @endif
     */
    virtual InPortConsumer* clone() const = 0;
    
    /*!
     * @if jp
     * @brief ǡμؤϿ
     *
     * ꤵ줿ץѥƥƤ˴ŤơǡΤμϿ
     * ν貾۴ؿ
     *
     * @param properties Ͽ˻Ȥץѥƥ
     *
     * @return Ͽ
     *
     * @else
     * @brief Subscribe the data send notification
     *
     * Pure virtual function to subscribe the data send notification
     * based on specified property information.
     *
     * @param properties Properties for reference when subscribing
     *
     * @return Subscription result
     *
     * @endif
     */
    virtual bool subscribeInterface(const SDOPackage::NVList& properties) = 0;
    
    /*!
     * @if jp
     * @brief ǡμ꤫Ͽ
     *
     * ǡΤμ꤫Ͽ뤿ν貾۴ؿ
     *
     * @param properties Ͽ˻Ȥץѥƥ
     *
     * @else
     * @brief Unsubscribe the data send notification
     *
     * Pure virtual function to unsubscribe the data send notification.
     *
     * @param properties Properties for reference when unsubscribing
     *
     * @endif
     */
    virtual void unsubscribeInterface(const SDOPackage::NVList& properties) = 0;
    
  protected:
    
  private:
    
  };
};     // namespace RTC
#endif // InPortConsumer_h
