// -*- C++ -*-
/*!
 * @file OutPortBase.h
 * @brief InPortBase base class
 * @date $Date: 2007-12-31 03:08:05 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: OutPortBase.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef RtcOutPortBase_h
#define RtcOutPortBase_h

#include <vector>
#include <string>

namespace RTC
{
  class PublisherBase;
  /*!
   * @if jp
   *
   * @class OutPortBase
   *
   * @brief OutPort 쥯饹
   * 
   * OutPort μǤ OutPort<T> δ쥯饹
   *
   * OutPortBase  PublisherBase ϰ Observer ѥƤ롣
   * OutPortBase  attach(), detach(), notify() 
   * PublisherBase  push()  Observer ѥ˴Ϣ᥽åɤǤ롣
   *
   * @since 0.2.0
   *
   * @else
   *
   * @class OutPortBase
   *
   * @brief Output base class.
   *
   * The base class of OutPort<T> which are implementations of OutPort
   *
   * Form a kind of Observer pattern with OutPortBase and PublisherBase.
   * attach(), detach(), notify() of OutPortBase and
   * push() of PublisherBase are methods associated with the Observer pattern.
   *
   * @since 0.2.0
   *
   * @endif
   */
  class OutPortBase
  {
  public:
    /*!
     * @if jp
     * @brief 󥹥ȥ饯
     *
     * 󥹥ȥ饯
     *
     * @param name ݡ̾
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @param name Port name
     *
     * @endif
     */
    OutPortBase(const char* name)
      : m_name(name)
    {
    };
    
    /*!
     * @if jp
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     * Ͽ줿Ƥ Publisher 롣
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     * Detach all subscribed Publishers.
     *
     * @endif
     */
    virtual ~OutPortBase();
    
    /*!
     * @if jp
     * @brief OutPort̾Τμ
     *
     * OutPort̾Τ롣
     *
     * @return ݡ̾
     *
     * @else
     *
     * @brief Retrieve OutPort name
     *
     * Retrieve OutPort name
     *
     * @return the port name
     *
     * @endif
     */
    inline const char* name() {return m_name.c_str();}
    
    /*!
     * @if jp
     * @brief Publisherɲ
     *
     * ꤷPublisherǡȤƥꥹȤκǸɲä롣
     * attach_back() Ʊͤʵǽ
     *
     * @param id ꤵ줿Publisher˳ƤID
     * @param publisher ϿоPublisher֥
     *
     * @else
     *
     * @brief Attach the publisher
     *
     * Attach the specified publisher as the destination of the data update
     * notification to the end of the Publisher list.
     * This function is similar to attach_back().
     *
     * @param id ID assigned to the Publisher that is specified
     * @param publisher The target Publisher object for attachment
     *
     * @endif
     */
    void attach(const char* id, PublisherBase* publisher);
    
    /*!
     * @if jp
     * @brief ꥹƬؤPublisherɲ
     *
     * PublisherꥹȤƬɲä롣
     *
     * @param id ꤵ줿Publisher˳ƤID
     * @param publisher ϿоPublisher֥
     *
     * @else
     *
     * @brief Attach the publisher to the top of the Publisher list.
     *
     * Attach the publisher to the top of the Publisher list.
     *
     * @param id ID assigned to the Publisher that is specified
     * @param publisher The target Publisher object for attachment
     *
     * @endif
     */
    void attach_front(const char* id, PublisherBase* publisher);
    
    /*!
     * @if jp
     * @brief ꥹȺǸؤPublisherɲ
     *
     * PublisherꥹȤκǸɲä롣
     *
     * @param id ꤵ줿Publisher˳ƤID
     * @param publisher ϿоPublisher֥
     *
     * @else
     *
     * @brief Attach the publisher at the end of the Publisher list.
     *
     * Attach the publisher to the end of the Publisher list.
     *
     * @param id ID assigned to the Publisher that is specified
     * @param publisher The target Publisher object for attachment
     *
     * @endif
     */
    void attach_back(const char* id, PublisherBase* publisher);
    
    /*!
     * @if jp
     * @brief Publisherκ
     *
     * ꤵ줿 Publisher ǡꥹȤ롣
     *
     * @param id о Publisher ID
     *
     * @return ϡ Publisher ֥
     *         ꤷ Publisher ¸ߤʤ null
     *
     * @else
     *
     * @brief Detach the publisher
     *
     * Detach the specified Publisher from the update notification list
     *
     * @param id ID of the deleted Publisher
     *
     * @return The deleted publisher object if the deletion completed
     *         successfully.
     *         Null if the specified Publisher does not exist.
     *
     * @endif
     */
    PublisherBase* detach(const char* id);
    
    /*!
     * @if jp
     * @brief 
     *
     * ϿƤƤ Publisher ˥ǡΤ롣
     *
     * @else
     *
     * @brief Notify the data updates
     *
     * Notify data updates to all registered Publishers
     *
     * @endif
     */
    void notify();
    
  protected:
    /*!
     * @if jp
     * @brief ݡ̾
     * @else
     * @brief Port name
     * @endif
     */
    std::string m_name;
    
    /*!
     * @if jp
     * @brief Publisher ѹ¤
     * @else
     * @brief Structure of Publisher
     * @endif
     */
    struct Publisher
    {
      Publisher(const char* _id, PublisherBase* _publisher)
	: id(_id), publisher(_publisher){};
      std::string id;
      PublisherBase* publisher;
    };
    
    /*!
     * @if jp
     * @brief Ͽ줿 Publisher ꥹ
     * @else
     * @brief Registered Publisher list
     * @endif
     */
    std::vector<Publisher*> m_publishers;
    
    /*!
     * @if jp
     * @brief Publisher  ID ˤäƸ뤿 Functor
     * @else
     * @brief Functor to find Publisher by id
     * @endif
     */
    // Functor to find Publisher by id
    struct find_id;
    
    /*!
     * @if jp
     * @brief Publisher ˥ǡΤ뤿 Functor
     * @else
     * @brief Functor to notify updates to Publishers
     * @endif
     */
    // Functor to notify update to Publishers
    struct pub_push;
    
    /*!
     * @if jp
     * @brief Publisher 뤿 Functor
     * @else
     * @brief Functor to delete Publishers
     * @endif
     */
    // Functor to delete Publishers
    struct pub_del;
  };
}; // End of namespace RTM
#endif // RtcOutPortBase_h
