// -*- C++ -*-
/*!
 * @file  PublisherFactory.h
 * @brief PublisherFactory class
 * @date  $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PublisherFactory.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef PublisherFactory_h
#define PublisherFactory_h

#include <rtm/Properties.h>

namespace RTC
{
  class PublisherBase;
  class InPortConsumer;
  class Properties;
  
  /*!
   * @if jp
   * @class PublisherFactory
   * @brief PublisherFactory 饹
   *
   * ƼPublisher֥Ȥ˴եȥꥯ饹
   * ƥݥʼ
   *   ŪˤǤդPublisherǤ褦ˤ롣
   *
   * @else
   * @class PublisherFactory
   * @brief PublisherFactory class
   *
   * This is a factory class to manage for creation and destruction of
   * various Publisher objects.
   * Note: This is temporary implementation.
   *  We try to be able to generate any Publisher in the future.
   *
   * @endif
   */
  class PublisherFactory
  {
  public:
    /*!
     * @if jp
     * @brief 󥹥ȥ饯
     *
     * ǥեȥ󥹥ȥ饯
     *
     * @else
     * @brief Constructor
     *
     * Default constructor
     *
     * @endif
     */
    PublisherFactory(){};
    
    /*!
     * @if jp
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~PublisherFactory(){};
    
    /*!
     * @if jp
     * @brief Publisher
     *
     * Publisher֥Ȥ롣
     * ꤵ줿˱ŬڤPublisherΥ֥Ȥ롣
     * Publisherμ̤򡢻ꤵ줿Property֥Ȥ
     * dataport.subscription_typeФꤷƤɬפ롣
     * ޤ̤ˤäƤϡPublisherζư椹򤵤ꤹɬפ
     * 롣
     * ζŪƤϡ줾Publisher򻲾ȤΤȡ
     *
     * @param consumer PublisherˤäƥǡФư륳󥷥塼
     * @param property ٤Publisherꤹ뤿ξ䡢Publisher
     *                 ư椹뤿ξꤵƤProperty֥
     *                 
     *
     * @return Publisher֥ȡ˼ԤNull֤
     *
     * @else
     * @brief Create the Publisher
     *
     * Create the Publisher's object.
     * The appropriate Publisher implementation object according to the
     * specified argument is generated.
     * The type of generated Publisher needs to be set to the 
     * dataport.subscription_type member of the specified Property object.
     * Also, depending on the type, information that controls the behavior of
     * Publisher needs to be set.
     * For more details on these contents, please refer to each Publisher 
     * implementation.
     *
     * @param consumer Consumer that is driven the data sending by Publisher
     * @param property Property object to be set information for specifying to
     *                 the generated Publisher and for controlling the drive of
     *                 Publisher.
     *
     * @return The generated Publisher's object. Null will be returned if failed
     *         to be generated.
     *
     * @endif
     */
    PublisherBase* create(InPortConsumer* consumer, Properties& property);
    
    /*!
     * @if jp
     * @brief Publisher˴
     *
     * ꤵ줿Publisher֥Ȥ˴롣
     *
     * @param publisher ˴оPublisher֥
     *
     * @else
     * @brief Destroy the Publisher
     *
     * Destroy the specified Publisher's object
     *
     * @param publisher The target Publisher's object for the delete
     *
     * @endif
     */
    void destroy(PublisherBase *publisher);
  protected:
    
  private:
    
  };
};     // namespace RTC
#endif // PublisherFactory_h

