// -*- C++ -*-
/*!
 * @file  PublisherNew.h
 * @brief PublisherNew class
 * @date  $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PublisherNew.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef PublisherNew_h
#define PublisherNew_h

#include <rtm/RTC.h>

#include <ace/Task.h>
#include <ace/Synch.h>
#include <rtm/PublisherBase.h>

namespace RTC
{
  class InPortConsumer;
  class Properties;
  /*!
   * @if jp
   * @class PublisherNew
   * @brief PublisherNew 饹
   *
   * Хåե˿ǡǼ줿ߥ󥰤ǡοǡ롣
   * ǡХߥ󥰤Ԥĥ󥷥塼ޤФ¦Ȥϰۤʤ륹åɤ
   * ư˻ѡ
   * PublisherζưϡǡФΥߥ󥰤ˤʤޤǥ֥å졢
   * Хߥ󥰤Τȡ¨¤˥󥷥塼ޤнƤӽФ
   *
   * @else
   * @class PublisherNew
   * @brief PublisherNew class
   *
   * Send new data at timing of when it is stored into the buffer.
   * This class is used when operating Consumer that waits for the data send
   * timing in different thread from one of the send side.
   * Publisher's driven is blocked until the data send timing reaches, if the
   * send timing notification is received, the Consumer's send processing will
   * be invoked immediately.
   *
   * @endif
   */
  class PublisherNew
    : public PublisherBase,
      public ACE_Task<ACE_MT_SYNCH>
  {
  public:
    /*!
     * @if jp
     * @brief 󥹥ȥ饯
     *
     * 󥹥ȥ饯
     *  Publisher ѿåɤ롣
     *
     * @param consumer ǡФԤĥ󥷥塼
     * @param property PublisherζưꤷProperty֥
     *                 (PublisherǤ̤)
     * @else
     * @brief Constructor
     *
     * Constructor
     * Create new thread for this Publisher.
     *
     * @param consumer Consumer that waits for the data sending
     * @param property Property object that is configured this Publisher's
     *                 control information.(Unused in this Publisher)
     * @endif
     */
    PublisherNew(InPortConsumer* consumer,
		 const Properties& property);
    
    /*!
     * @if jp
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~PublisherNew();
    
    /*!
     * @if jp
     * @brief Observerؿ
     *
     * Хߥ󥰻˸ƤӽФ
     * ֥åƤPublisherζưϤ졢󥷥塼ޤؤн
     * Ԥ롣
     *
     * @else
     * @brief Observer function
     *
     * Invoke at send timing.
     * Start this Publisher's control that has been blocked and do the send
     * processing to Consumer.
     *
     * @endif
     */
    virtual void update();
    
    /*!
     * @if jp
     * @brief åɼ¹Դؿ
     *
     * ACE_Task::svc() ΥС饤
     * ХåեΥǡޤǥåɤԵ롣
     *
     * @return ¹Է
     *
     * @else
     * @brief Thread execution function
     *
     * ACE_Task::svc() override function.
     * Make thread stand by until data in the buffer is updated.
     *
     * @return Execution result
     *
     * @endif
     */
    virtual int svc(void);
    
    /*!
     * @if jp
     * @brief 
     *
     * ACE_Task::open() ΥС饤
     *  publisher ѿåɤ롣
     *
     * @param args åѰ(PublisherǤ̤)
     *
     * @return ¹Է
     *
     * @else
     * @brief Task start function
     *
     * ACE_Task::open() override function.
     * Create newly thread for this Publisher.
     *
     * @param args Thread creation arguments (Unused in this Publisher)
     *
     * @return Execution result
     *
     * @endif
     */
    virtual int open(void *args);
    
    /*!
     * @if jp
     * @brief λؿ
     *
     * ACE_Task::release() ΥС饤
     * ưե饰falseꤷ Publisher ζưߤ롣
     * ưåɤ֥åƤˤϡ
     * 磱󥳥󥷥塼ޤнƤӽФ礬롣
     *
     * @else
     * @brief Task terminate function
     *
     * ACE_Task::release() override function.
     * Set driven flag to false, and terminate this Publisher's operation.
     * However, if the driven thread is blocked, Consumer's send
     * processing may be invoked maximum once.
     *
     * @endif
     */
    virtual void release();
    
  protected:
    
  private:
    InPortConsumer* m_consumer;
    bool m_running;
    unsigned long m_usec;
    
    // NewData condition struct
    struct NewData
    {
      NewData() : _cond(_mutex), _updated(false) {};
      ACE_Thread_Mutex _mutex;
      ACE_Condition<ACE_Thread_Mutex> _cond;
      bool _updated;
    };
    
    // A condition variable for data update notification 
    NewData m_data;
  };
};     // namespace RTC
#endif // PublisherNew_h

