// -*- C++ -*-
/*!
 * @file   NVUtilTests.cpp
 * @brief  NVUtil test class
 * @date   $Date: 2006-11-27 08:26:13 $
 * @author Shinji Kurihara
 *         Noriaki Ando <n-ando@aist.go.jp>
 * 
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: NVUtilTests.cpp 726 2008-05-14 03:05:42Z n-ando $
 *
 */

#ifndef NVUtil_cpp
#define NVUtil_cpp

#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TextOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestAssert.h>

#include <rtm/NVUtil.h>

/*!
 * @class NVUtilTests class
 * @brief NVUtil test
 */
namespace NVUtil
{
  using namespace NVUtil;
  using namespace std;
  
  int g_argc;
  vector<string> g_argv;
  
  class NVUtilTests
    : public CppUnit::TestFixture
  {
    CPPUNIT_TEST_SUITE(NVUtilTests);
    CPPUNIT_TEST(test_newNV);
    //  CPPUNIT_TEST(test_newNVStr);
    CPPUNIT_TEST(test_newNVChar);
    CPPUNIT_TEST(test_newNVBool);
    CPPUNIT_TEST(test_newNVOctet);
    CPPUNIT_TEST(test_copy);
    CPPUNIT_TEST(test_find);
    CPPUNIT_TEST(test_isString);
    CPPUNIT_TEST(test_toString);
    CPPUNIT_TEST_SUITE_END();
    
  private:
    
  public:
    
    /*!
     * @brief Constructor
     */
    NVUtilTests()
    {
      CORBA::ORB_var orb;
      char* argv[g_argc];
      for (int i = 0; i < g_argc; i++) {
	argv[i] = (char *)g_argv[i].c_str();
      }
      
      orb = CORBA::ORB_init(g_argc, argv);
    }
    
    /*!
     * @brief Destructor
     */
    ~NVUtilTests()
    {
    }
    
    /*!
     * @brief Test initialization
     */
    virtual void setUp()
    {
    }
    
    /*!
     * @brief Test finalization
     */
    virtual void tearDown()
    { 
    }
    
    /*!
     * @brief newNV(char*,Value)Υƥ
     *   (1) newNV()ѤCORBA::ShortΥǡnameNameValue
     *   (2) newNV()ϤǡȼNameValueǤӡ
     *   (3) newNV()ѤCORBA::LongΥǡnameNameValue
     *   (4) newNV()ϤǡȼNameValueǤӡ
     *   (5) newNV()ѤCORBA::FloatΥǡnameNameValue
     *   (6) newNV()ϤǡȼNameValueǤӡ
     *   (7) newNV()ѤCORBA::DoubleΥǡnameNameValue
     *   (8) newNV()ϤǡȼNameValueǤӡ
     */
    void test_newNV() {
      CORBA::Short  setSt, getSt;
      CORBA::Long   setLg, getLg;
      CORBA::Float  setFt, getFt;
      CORBA::Double setDl, getDl;
      SDOPackage::NameValue nv;
      string setname, getname;
      
      // (1) newNV()ѤCORBA::ShortΥǡnameNameValue
      setSt = 1;
      nv = newNV("short", setSt);
      
      // (2) newNV()ϤǡȼNameValueǤӡ
      setname = "short";
      getname = nv.name;
      CPPUNIT_ASSERT(getname == setname);
      nv.value >>= getSt;
      CPPUNIT_ASSERT(setSt == getSt);
      
      
      
      // (3) newNV()ѤCORBA::LongΥǡnameNameValue
      setLg = 999999999;
      nv = newNV("long", setLg);
      
      // (4) newNV()ϤǡȼNameValueǤӡ
      setname = "long";
      getname = nv.name;
      CPPUNIT_ASSERT(getname == setname);
      nv.value >>= getLg;
      CPPUNIT_ASSERT(setLg == getLg);
      
      
      
      // (5) newNV()ѤCORBA::FloatΥǡnameNameValue
      setFt = 99999.9;
      nv = newNV("float", setFt);
      
      // (6) newNV()ϤǡȼNameValueǤӡ
      setname = "float";
      getname = nv.name;
      CPPUNIT_ASSERT(getname == setname);
      nv.value >>= getFt;
      CPPUNIT_ASSERT(setFt == getFt);
      
      
      
      // (7) newNV()ѤCORBA::DoubleΥǡnameNameValue
      setDl = 9999999.999;
      nv = newNV("double", setDl);
      
      // (8) newNV()ϤǡȼNameValueǤӡ
      setname = "double";
      getname = nv.name;
      CPPUNIT_ASSERT(getname == setname);
      nv.value >>= getDl;
      CPPUNIT_ASSERT(setDl == getDl);
    }
    
    
    /*!
     * @brief newNV(char*,char*)Υƥ
     *    (1) const char*namevaluenewNV()ϤNameValue롣
     */
    void test_newNVStr() {
      SDOPackage::NameValue nv;
      const char* name  = "string";
      const char* value = "string-data";
      // (1) const char*namevaluenewNV()ϤNameValue롣
      nv = newNV(name, value);
    }
    
    
    /*!
     * @brief newNVChar()Υƥ
     *   (1) CORBA::Charvaluechar*namenewNVChar()ϤNameValue롣
     *   (2) newNVChar()˥åȤnameȼNameValue.nameӡ
     *   (3) newNVChar()˥åȤvalueȼNameValue.valueӡ
     */
    void test_newNVChar() {
      CORBA::Char ch = 'A', getch;
      SDOPackage::NameValue nv;
      // (1) CORBA::Charvaluechar*namenewNVChar()ϤNameValue롣
      nv = newNVChar("char", ch);
      
      string setstr, getstr;
      
      // (2) newNVChar()˥åȤnameȼNameValue.nameӡ
      setstr = "char";
      getstr = nv.name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      // (3) newNVChar()˥åȤvalueȼNameValue.valueӡ
      nv.value >>= CORBA::Any::to_char(getch);
      CPPUNIT_ASSERT(ch == getch);
    }
    
    
    /*!
     * @brief newNVBool()Υƥ
     */
    void test_newNVBool() {
      SDOPackage::NameValue nv;
      CORBA::Boolean setval=false, retval;
      
      nv = newNVBool("bool", setval);
      
      string setstr, getstr;
      
      setstr = "bool";
      getstr = nv.name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      nv.value >>= CORBA::Any::to_boolean(retval);
      CPPUNIT_ASSERT(setval == retval);
    }
    
    
    /*!
     * @brief newNVOctet()Υƥ
     */
    void test_newNVOctet() {
      SDOPackage::NameValue nv;
      CORBA::Octet setval=030, getval;
      
      nv = newNVOctet("octet", setval);
      
      string setstr, getstr;
      
      setstr = "octet";
      getstr = nv.name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      nv.value >>= CORBA::Any::to_octet(getval);
      CPPUNIT_ASSERT(setval == getval);
    }
    
    /*!
     * @brief copy()Υƥ
     *   (1) RTC::Properties֥Ȥ
     *   (2copy()ˤProperties֥Ȥnvlist˥ԡ
     *   (3) NVList[0].valueǡ
     *   (4) copy()ˤͿnvlist񤭴Ƥ뤫ǧ
     */
    void test_copy() {
      SDOPackage::NVList nvlist;
      
      map<string, string> mProp;
      // (1) RTC::Properties֥Ȥ
      //  PropertiesΥ󥹥ȥ饯ǤϰͿ줿mapΥͤ¸롣
      mProp["potr-type"];
      RTC::Properties prop(mProp);
      
      
      // (2copy()ˤProperties֥Ȥnvlist˥ԡ
      copy(nvlist, prop);
      
      
      // (3) NVList[0].valueǡ
      const char* getval;
      nvlist[0].value >>= getval;
      
      string setstr, getstr;
      
      
      // (4) copy()ˤͿnvlist񤭴Ƥ뤫ǧ
      setstr = "potr-type";
      getstr = nvlist[0].name;
      CPPUNIT_ASSERT(setstr == getstr);
      
      setstr = "";
      getstr = getval;
      CPPUNIT_ASSERT(setstr == getstr);
      
    }
    
    
    /*!
     * @brief find()Υƥ
     *   (1) NVListǤname"short",valueshortΥǡ򥻥åȡ
     *   (2) NVListǤname"long",valuelongΥǡ򥻥åȡ
     *   (3) nvlist椫NameValue.name"long"NameValue.value
     *   (4) nvlist椫NameValue.name"short"NameValue.value
     */
    void test_find() {
      SDOPackage::NVList nvlist;
      CORBA::Any any;
      CORBA::Short setst,getst;
      CORBA::Long  setlg,getlg;
      
      nvlist.length(2);
      
      // (1) NVListǤname"short",valueshortΥǡ򥻥åȡ
      setst = 1;
      nvlist[0].name = "short";
      nvlist[0].value <<= setst;
      
      // (2) NVListǤname"long",valuelongΥǡ򥻥åȡ
      setlg =111;
      nvlist[1].name = "long";
      nvlist[1].value <<= setlg;
      
      // (3) nvlist椫NameValue.name"long"NameValue.value
      any = find(nvlist, "long");
      any >>= getlg;
      CPPUNIT_ASSERT(setlg == getlg);
      
      // (4) nvlist椫NameValue.name"short"NameValue.value
      any = find(nvlist, "short");
      any >>= getst;
      CPPUNIT_ASSERT(setst == getst);
    }
    
    
    /*!
     * @brief isString()Υƥ
     *   (1) NVListǤname"short",valueshortΥǡ򥻥åȡ
     *   (2) NVListǤname"string",valuestringΥǡ򥻥åȡ
     *   (3) isString(nvlist,name)ˤ,ꤵ줿namevalueηstringɤȽꡣ
     *   (4) isString(nvlist,name)ˤ,ꤵ줿namevalueηstringɤȽꡣ
     */
    void test_isString() {
      bool result;
      SDOPackage::NVList nvlist;
      CORBA::Short setst;
      
      nvlist.length(2);
      
      // (1) NVListǤname"short",valueshortΥǡ򥻥åȡ
      setst = 1;
      nvlist[0].name = "short";
      nvlist[0].value <<= setst;
      
      // (2) NVListǤname"string",valuestringΥǡ򥻥åȡ
      string str("test");
      nvlist[1].name = "string";
      nvlist[1].value <<= str.c_str();
      
      // (3) isString(nvlist,name)ˤ,ꤵ줿namevalueηstringɤȽꡣ
      result = isString(nvlist, "string");
      CPPUNIT_ASSERT(result == true);
      
      // (4) isString(nvlist,name)ˤ,ꤵ줿namevalueηstringɤȽꡣ
      result = isString(nvlist, "short");
      CPPUNIT_ASSERT(result == false);
      
    }
    
    
    /*!
     * @brief toString()Υƥ
     *   (1) NVListǤname"short",valueshortΥǡ򥻥åȡ
     *   (2) NVListǤname"string",valuestringΥǡ򥻥åȡ
     *   (3) toString(nvlist,name)ˤ,ꤵ줿namevaluestringǼ
     *   (4) toString(nvlist,name)ˤ,ꤵ줿namevaluestringǼ
     */
    void test_toString() {
      string result;
      SDOPackage::NVList nvlist;
      CORBA::Short setst;
      
      nvlist.length(2);
      
      // (1) NVListǤname"short",valueshortΥǡ򥻥åȡ
      setst = 1;
      nvlist[0].name = "short";
      nvlist[0].value <<= setst;
      
      // (2) NVListǤname"string",valuestringΥǡ򥻥åȡ
      string str("test"), setstr;
      nvlist[1].name = "string";
      nvlist[1].value <<= str.c_str();
      
      // (3) toString(nvlist,name)ˤ,ꤵ줿namevaluestringǼ
      result = toString(nvlist, "string");
      setstr = "test";
      CPPUNIT_ASSERT(result == setstr);
      
      // Failure case:
      // 
      // (4) toString(nvlist,name)ˤ,ꤵ줿namevaluestringǼ
      result = toString(nvlist, "short");
      setstr = "";
      CPPUNIT_ASSERT(result == setstr);
    }
    
  };
}; // namespace NVUtil

/*
 * Register test suite
 */
CPPUNIT_TEST_SUITE_REGISTRATION(NVUtil::NVUtilTests);

#ifdef LOCAL_MAIN
int main(int argc, char* argv[])
{
  CppUnit::TextUi::TestRunner runner;
  runner.addTest(CppUnit::TestFactoryRegistry::getRegistry().makeTest());
  CppUnit::Outputter* outputter = 
    new CppUnit::TextOutputter(&runner.result(), std::cout);
  runner.setOutputter(outputter);
  bool retcode = runner.run();
  return !retcode;
}
#endif // MAIN
#endif // NVUtil_cpp
