// -*- C++ -*-
/*!
 * @file   PortAdminTests.cpp
 * @brief  PortAdmin test class
 * @date   $Date: 2007-04-13 15:05:03 $
 * @author Shinji Kurihara
 *         Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PortAdminTests.cpp 726 2008-05-14 03:05:42Z n-ando $
 *
 */

#ifndef PortAdmin_cpp
#define PortAdmin_cpp

#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TextOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestAssert.h>

#include <vector>
#include <string>

#include <rtm/PortAdmin.h>

/*!
 * @class PortAdminTests class
 * @brief PortAdmin test
 */
namespace PortAdmin
{
  using namespace RTC;
  using namespace std;
  
  int g_argc;
  vector<string> g_argv;
  
  class PortBase
	: public RTC::PortBase
  {
  protected:
	virtual RTC::ReturnCode_t publishInterfaces(RTC::ConnectorProfile&)
	{
	  return RTC::RTC_OK;
	}
	virtual RTC::ReturnCode_t subscribeInterfaces(const RTC::ConnectorProfile&)
	{
	  return RTC::RTC_OK;
	}
	virtual void unsubscribeInterfaces(const RTC::ConnectorProfile&)
	{
	}
  };


  class PortAdminTests
    : public CppUnit::TestFixture
  {
    CPPUNIT_TEST_SUITE(PortAdminTests);
    CPPUNIT_TEST(test_getPortList);
    CPPUNIT_TEST(test_getPortRef);
    CPPUNIT_TEST(test_getPort);
    CPPUNIT_TEST(test_registerPort);
    CPPUNIT_TEST(test_deletePort);
    CPPUNIT_TEST(test_deletePortByName);
    CPPUNIT_TEST(test_finalizePorts);
    CPPUNIT_TEST_SUITE_END();
    
  private:
	RTC::PortAdmin* m_ppadm;
	PortBase* m_ppb;
	PortBase* m_ppb2;
    CORBA::ORB_ptr          m_orb;
    PortableServer::POA_ptr m_poa;
    
  public:
    
    /*!
     * @brief Constructor
     */
    PortAdminTests()
    {
      char* argv[g_argc];
      for (int i = 0; i < g_argc; i++) {
	argv[i] = (char *)g_argv[i].c_str();
      }
      
      m_orb = CORBA::ORB_init(g_argc, argv);
      CORBA::Object_var  obj = m_orb->resolve_initial_references("RootPOA");
      m_poa = PortableServer::POA::_narrow(obj);
      PortableServer::POAManager_var pman = m_poa->the_POAManager();
      pman->activate();
    }
    
    /*!
     * @brief Destructor
     */
    ~PortAdminTests()
    {
    }
    
    /*!
     * @brief /
     * ǤϰʲνԤäƤ롣
     *  (1) PortAdmin饹PortBase饹Υ󥹥
     *  (2) PortBase饹PortProfileϿ
     *  (3) PortAdmin饹PortBase֥ȤϿ
     */
    virtual void setUp()
    {
      // PortAdmin饹Υ󥹥
      m_ppadm = new RTC::PortAdmin(m_orb, m_poa);
      
      // PortBase饹Υ󥹥
      m_ppb = new PortBase();
      
      // PortBase饹Υ󥹥
      m_ppb2 = new PortBase();
      
      // PortProfileϿ
      m_ppb->setName("port0");
      m_ppb2->setName("port1");
      
      // PortBase֥ȤϿ
      m_ppadm->registerPort(*m_ppb);
      m_ppadm->registerPort(*m_ppb2);
    }
    
    /*!
     * @brief Test finalization
     */
    virtual void tearDown()
    { 
      //    delete m_ppb;
      // PortAdmin󥹥󥹤˴
      delete m_ppadm;
    }
    
    /*!
     * @brief getPortList()Υƥ
     *    getPortList()PortListΥݥ󥿤Υݥ󥿤ѤPort󥿥ե
     *    ڥ졼ƤӽФԤäƤ롣
     *    PortListϻsetUp()᥽åɤϿƤ롣
     */
    void test_getPortList() {
      PortList* getPList;
      // getPortList()Υƥ
      getPList = m_ppadm->getPortList();
      
      PortProfile *getProf0, *getProf1;
      // Port󥿥եΥڥ졼ƤӽФ
      getProf0 = (*getPList)[0]->get_port_profile();
      
      string setstr, getstr;
      getstr = getProf0->name;
      setstr = "port0";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getProf1 = (*getPList)[1]->get_port_profile();
      getstr = getProf1->name;
      setstr = "port1";
      CPPUNIT_ASSERT(getstr == setstr);
    }
    
    
    /*!
     * @brief getPortRef()Υƥ
     *   (1) get_PortRef()PortΥ֥ȥե󥹼
     *   (2) ֥ȥե󥹤ѤPortڥ졼θƤӽФԤ
     *    PortΥ֥ȥե󥹤setUp()ˤϿƤ롣
     */
    void test_getPortRef() {
      Port_var getP;
      string getstr, setstr;
      PortProfile *getProf;
      
      //========= Failure case ================
      // ϿƤʤPortProfile.namegetPortRef()Ƥ֤nill֤롣OK.
      //    getP = m_ppadm->getPortRef("");
      //    if (CORBA::is_nil(getP))
      //      cout << "getP is nil." << endl;
      //    getP = m_ppadm->getPortRef("test");
      //    if (CORBA::is_nil(getP))
      //      cout << "getP is nil." << endl;
      //========================================================
      
      // getPortRef()Υƥ
      getP = m_ppadm->getPortRef("port1");
      
      if (CORBA::is_nil(getP))
	cout << "getP is nil." << endl;
      
      getProf = getP->get_port_profile();
      getstr = getProf->name;
      setstr = "port1";
      CPPUNIT_ASSERT(getstr == setstr);
      
      
      // test  getPortRef()
      getP = m_ppadm->getPortRef("port0");
      
      if (CORBA::is_nil(getP))
	cout << "getP is nil." << endl;
      
      getProf = getP->get_port_profile();
      getstr = getProf->name;
      setstr = "port0";
      CPPUNIT_ASSERT(getstr == setstr);
    }
    
    
    /*!
     * @brief getPort()Υƥ
     *   (1) getPort()ˤPortBaseؤΥݥ󥿤
     *   (2) ݥ󥿤ѤPortBase饹Υ᥽åɸƤӽФԤ
     *    PortBase饹Υݥ󥿤setUp()ϿѤߤǤ롣
     */
    void test_getPort() {
	  RTC::PortBase* pb;
      PortProfile* getProf;
      string setstr, getstr;
      
      // Failure case: unknown exception㳰ȯ
      //    pb = m_ppadm->getPort("");
      //    pb = m_ppadm->getPort("test"); // ϿƤʤname
      
      // test getPort()
      pb = m_ppadm->getPort("port1");
      getProf = pb->get_port_profile();
      setstr = "port1";
      getstr = getProf->name;
      CPPUNIT_ASSERT(setstr == getstr);
      
      // test getPort()
      pb = m_ppadm->getPort("port0");
      getProf = pb->get_port_profile();
      setstr = "port0";
      getstr = getProf->name;
      CPPUNIT_ASSERT(setstr == getstr);
    }
    
    
    /*!
     * @brief tests for registerPort()
     */
    void test_registerPort() {
      // setUp()ˤƥƥȡ
    }
    
    
    /*!
     * @brief deletePort()Υƥ
     *    Port_ptrsetUp()ˤregisterPort()ѤϿѤߤǤ롣(2ĤΥݡȤϿ)
     *   (1) deletePort()ƤӽФ
     *   (2) getPortList()ˤPortList
     *   (3) (1)νԤƤ뤫ǧ
     */
    void test_deletePort() {
      
      // (1) deletePort()ƤӽФ
      m_ppadm->deletePort(*m_ppb2);
      
      // (2) getPortList()ˤPortList
      PortList* getPList;
      getPList = m_ppadm->getPortList();
      
      cout << getPList->length() << endl;
      
      
      // (3) (1)νԤƤ뤫ǧ
      PortProfile *getProf0, *getProf1;
      // PortBase֥ȡݥڥ졼ƤӽФ
      getProf0 = (*getPList)[0]->get_port_profile();
      string setstr, getstr;
      getstr = getProf0->name;
      setstr = "port0";
      CPPUNIT_ASSERT(getstr == setstr);
      
      
      // Failure case :
      //    setUp()2ĤΥݡȤϿƤꡢ(1)ǰĺΤǡǤν
      //    顼ȤʤϤǤ롣
      getProf1 = (*getPList)[1]->get_port_profile();
      getstr = getProf1->name;
      setstr = "port1";
      CPPUNIT_ASSERT(getstr == setstr);
    }
    
    
    /*!
     * @brief tests for deletePortByName()
     *    Port_ptrsetUp()ˤregisterPort()ѤϿѤߤǤ롣(2ĤΥݡȤϿ)
     *    (1) getPortList()ˤPortListϿƤPortοǧ
     *    (2deletePortByName()ˤ"port1"̾PortκԤ
     *    (3) getPortList()ˤPortListϿƤPortοǧ
     */
    void test_deletePortByName() {
      PortList* getPList;
      
      // (1) getPortList()ˤPortListϿƤPortοǧ
      getPList = m_ppadm->getPortList();
      cout << getPList->length() << endl;
      
      m_ppadm->deletePortByName("port1");
      
      // (3) getPortList()ˤPortListϿƤPortοǧ
      getPList = m_ppadm->getPortList();
      cout << getPList->length() << endl;
    }
    
    
    /*!
     * @brief tests for finalizePorts()
     *   (1) getPortList()ˤPortListϿƤPortοǧ
     *   (2) finalizePorts()θƤӽФ
     *   (3) getPortList()ˤPortListϿƤPortοǧ
     */
    void test_finalizePorts() {
      PortList* getPList;
      
      // (1) getPortList()ˤPortListϿƤPortοǧ
      getPList = m_ppadm->getPortList();
      cout << getPList->length() << endl;
      
      
      // (2) finalizePorts()θƤӽФ
      m_ppadm->finalizePorts();
      
      
      // (3) getPortList()ˤPortListϿƤPortοǧ
      getPList = m_ppadm->getPortList();
      cout << getPList->length() << endl;
    }
    
    
  };
}; // namespace PortAdmin

/*
 * Register test suite
 */
CPPUNIT_TEST_SUITE_REGISTRATION(PortAdmin::PortAdminTests);

#ifdef LOCAL_MAIN
int main(int argc, char* argv[])
{
  CppUnit::TextUi::TestRunner runner;
  runner.addTest(CppUnit::TestFactoryRegistry::getRegistry().makeTest());
  CppUnit::Outputter* outputter = 
    new CppUnit::TextOutputter(&runner.result(), std::cout);
  runner.setOutputter(outputter);
  bool retcode = runner.run();
  return !retcode;
}
#endif // MAIN
#endif // PortAdmin_cpp
