// -*- C++ -*-
/*!
 * @file   PortBaseTests.cpp
 * @brief  PortBase test class
 * @date   $Date: 2007-04-13 15:05:10 $
 * @author Shinji Kurihara
 *         Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PortBaseTests.cpp 726 2008-05-14 03:05:42Z n-ando $
 *
 */

#ifndef PortBase_cpp
#define PortBase_cpp

#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TextOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestAssert.h>

#include <vector>
#include <string>
#include <rtm/PortBase.h>
#include <rtm/RTObject.h>

/*!
 * @class PortBaseTests class
 * @brief PortBase test
 */
namespace PortBase
{
  class PortBase
	: public RTC::PortBase
  {
  public:
	const std::string getUUID() const
	{
	  return RTC::PortBase::getUUID();
	}
  protected:
	virtual RTC::ReturnCode_t publishInterfaces(RTC::ConnectorProfile&)
	{
	  return RTC::RTC_OK;
	}
	virtual RTC::ReturnCode_t subscribeInterfaces(const RTC::ConnectorProfile&)
	{
	  return RTC::RTC_OK;
	}
	virtual void unsubscribeInterfaces(const RTC::ConnectorProfile&)
	{
	}
  };

  using namespace RTC;
  using namespace std;
  int g_argc;
  vector<string> g_argv;

  class PortBaseTests
    : public CppUnit::TestFixture
  {
    CPPUNIT_TEST_SUITE(PortBaseTests);
    CPPUNIT_TEST(test_get_port_profile);
    CPPUNIT_TEST(test_get_connector_profiles);
    CPPUNIT_TEST(test_get_connector_profile);
    CPPUNIT_TEST(test_connect);
    CPPUNIT_TEST(test_disconnect);
    CPPUNIT_TEST(test_disconnect_all);
    //  CPPUNIT_TEST(test_setProfile);
    //  CPPUNIT_TEST(test_getProfile);
    //    CPPUNIT_TEST(test_setName);
    //    CPPUNIT_TEST(test_getName);
    //    CPPUNIT_TEST(test_setInterfaceProfiles);
    //  CPPUNIT_TEST(test_addInterfaceProfiles);
    //  CPPUNIT_TEST(test_getInterfaceProfiles);
    //  CPPUNIT_TEST(test_getInterfaceProfile);
    CPPUNIT_TEST(test_setPortRef);
    //  CPPUNIT_TEST(test_getPortRef);
    //  CPPUNIT_TEST(test_addConnectorProfile);
    //  CPPUNIT_TEST(test_eraseConnectorProfile);
    //  CPPUNIT_TEST(test_getConnectorProfileList);
    //  CPPUNIT_TEST(test_getConnectorProfile);
    CPPUNIT_TEST(test_setOwner);
    CPPUNIT_TEST(test_getOwner);
	//    CPPUNIT_TEST(test_setProperties);
	//    CPPUNIT_TEST(test_getProperties);
    CPPUNIT_TEST(test_getUUID);
    CPPUNIT_TEST_SUITE_END();
    
  private:
    PortBase* m_ppb;
    CORBA::ORB_ptr          m_orb;
    PortableServer::POA_ptr m_poa;
    
    SDOPackage::NVList m_nvlist;
    SDOPackage::NameValue m_nv;
    CORBA::Float m_cnctProfVal,m_portProfVal;
  public:
    
    /*!
     * @brief Constructor
     */
    PortBaseTests()
    {
    }
    
    /*!
     * @brief Destructor
     */
    ~PortBaseTests()
    {
    }
    
    /*!
     * @brief 
     *    (1) ORBν,POAactivate
     *    (2) PortBaseΥ󥹥
     *    (3) PortInterfaceProfile֥ǤΥå
     *    (4) PortInterfaceProfileList֥
     *        (PortProfile)Υå
     *    (5) ConnectorProfile֥ǤΥå
     *    (6) ConnectorProfileList֥(PortProfile)Υå
     *    (7) PortProfile֥ǤΥå
     *    (8) PortProfile֥ȤΥå
     */
    virtual void setUp()
    {
      char* argv[g_argc];
      for (int i = 0; i < g_argc; i++) {
	argv[i] = (char *)g_argv[i].c_str();
      }
      
      // (1) ORBν,POAactivate
      m_orb = CORBA::ORB_init(g_argc, argv);
      CORBA::Object_var  obj = m_orb->resolve_initial_references("RootPOA");
      m_poa = PortableServer::POA::_narrow(obj);
      PortableServer::POAManager_var pman = m_poa->the_POAManager();
      pman->activate();
      
      // (2) PortBaseΥ󥹥
      m_ppb = new PortBase();
      
      // (3) PortInterfaceProfile֥(PortProfile)Υå
      PortInterfaceProfile pIProf;
      pIProf.instance_name = "PortInterfaceProfile-instance_name";
      pIProf.type_name = "PortInterfaceProfile-type_name";
      pIProf.polarity = REQUIRED;
      
      // (4) PortInterfaceProfileList֥ǤΥå
      PortInterfaceProfileList pIFProfList;
      pIFProfList.length(1);
      pIFProfList[0] = pIProf;
      
      // (5) ConnectorProfile֥ǤΥå
      ConnectorProfile cProf;
      cProf.name = "ConnectorProfile-name";
      cProf.connector_id = "connect_id0";
      m_nv.name = "ConnectorProfile-properties0-name";
      m_cnctProfVal = 1.1;
      m_nv.value <<= m_cnctProfVal;
      m_nvlist.length(1);
      m_nvlist[0] = m_nv;
      cProf.properties = m_nvlist;
      
      // (6) ConnectorProfileList֥(PortProfile)Υå
      ConnectorProfileList cProfList;
      cProfList.length(1);
      cProfList[0] = cProf;
      
      // (7) PortProfile֥ǤΥå
      PortProfile pProf;
      pProf.name = "inport0";
      pProf.interfaces = pIFProfList;
      pProf.connector_profiles = cProfList;
      m_nv.name = "PortProfile-properties0-name";
      m_portProfVal = 2.2;
      m_nv.value <<= m_portProfVal;
      m_nvlist[0] = m_nv;
      pProf.properties = m_nvlist;
      
      // (8) PortProfile֥ȤΥå
      //### m_ppb->setProfile(pProf);
    }
    
    /*!
     * @brief Test finalization
     */
    virtual void tearDown()
    { 
    }
    
    /*!
     * @brief get_port_profile()Υƥ
     *     PortProfilesetUp()ϿѤߤǤ롣
     *    (1) get_port_profile()ˤPortProfile*
     *    (2) åȤPortProfileȼPortProfileǤ
     */
    void test_get_port_profile() {
      PortProfile* getProf;
      
      // (1) get_port_profile()ˤPortProfile*
      getProf = m_ppb->get_port_profile();
      
      string setstr, getstr;
      // (2) åȤPortProfileȼPortProfileǤ
      // check PortProfile.name
      getstr = getProf->name;
      setstr = "inport0";
      CPPUNIT_ASSERT(getstr == setstr);
      
      // check PortProfile.interfaces
      getstr = getProf->interfaces[0].instance_name;
      setstr = "PortInterfaceProfile-instance_name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getProf->interfaces[0].type_name;
      setstr = "PortInterfaceProfile-type_name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      CPPUNIT_ASSERT(getProf->interfaces[0].polarity == REQUIRED);
      
      // check PortProfile.connector_profiles
      getstr = getProf->connector_profiles[0].name;
      setstr = "ConnectorProfile-name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getProf->connector_profiles[0].connector_id;
      setstr = "connect_id0";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getProf->connector_profiles[0].properties[0].name;
      setstr = "ConnectorProfile-properties0-name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      CORBA::Float retval;
      getProf->connector_profiles[0].properties[0].value >>= retval;
      CPPUNIT_ASSERT(retval == m_cnctProfVal);
      
      // check PortProfile.properties
      getstr = getProf->properties[0].name;
      setstr = "PortProfile-properties0-name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getProf->properties[0].value >>= retval;
      CPPUNIT_ASSERT(retval == m_portProfVal);
    }
    
    
    /*!
     * @brief get_connector_profiles()Υƥ
     *     ConnectorProfileListPortProfileǤǤꡢ
     *       setUp()ϿѤߤǤ롣
     *    (1get_connector_profiles()ˤConnectorProfileList
     *    (2) åȤConnectorProfileȼConnectorProfileList
     *        ǤǤConnectorProfileǤӡ
     */
    void test_get_connector_profiles() {
      ConnectorProfileList* cpList;
      string setstr, getstr;
      
      // get ConnectorProfileList
      cpList = m_ppb->get_connector_profiles();
      
      // (2) åȤConnectorProfileȼConnectorProfileList
      //     ǤǤ
      //      ConnectorProfileǤӡ
      // check ConnectorProfile.name
      setstr = "ConnectorProfile-name";
      getstr = (*cpList)[0].name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      // check ConnectorProfile.connector_id
      setstr = "connect_id0";
      getstr = (*cpList)[0].connector_id;
      CPPUNIT_ASSERT(getstr == setstr);
      
      // check ConnectorProfile.properties.name
      setstr = "ConnectorProfile-properties0-name";
      getstr = (*cpList)[0].properties[0].name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      // check ConnectorProfile.properties.value
      CORBA::Float retval;
      (*cpList)[0].properties[0].value >>= retval;
      CPPUNIT_ASSERT(m_cnctProfVal == retval);
    }
    
    
    /*!
     * @brief get_connector_profile()Υƥ
     *     ConnectorProfileConnectorProfileListǤǤꡢ
     *       ConnectorProfileListPortProfileǤǤ롣
     *       PortProfilesetUp()ϿѤߤǤ롣
     *    (1) get_connector_profileˤConnectorProfile
     *    (2) åȤConnectorProfileȼConnectorProfileӡ
     */
    void test_get_connector_profile() {
      ConnectorProfile* cProf;
      string setstr, getstr;
      
      // (1) get_connector_profileˤConnectorProfile
      cProf = m_ppb->get_connector_profile("connect_id0");
      
      // åȤConnectorProfileȼConnectorProfileӡ
      // check ConnectorProfile.name
      setstr = "ConnectorProfile-name";
      getstr = cProf->name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      // check ConnectorProfile.connector_id
      setstr = "connect_id0";
      getstr = cProf->connector_id;
      CPPUNIT_ASSERT(getstr == setstr);
      
      // check ConnectorProfile.properties.name
      setstr = "ConnectorProfile-properties0-name";
      getstr = cProf->properties[0].name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      // check ConnectorProfile.properties.value
      CORBA::Float retval;
      cProf->properties[0].value >>= retval;
      CPPUNIT_ASSERT(m_cnctProfVal == retval);
      
      ReturnCode_t result;
      
      result = m_ppb->connect(*cProf);
      
      if (result == OK)
	cout << "connect result OK. " << endl;
      else if (result == ERROR)
	cout << "connect result ERROR. " << endl;
      else if (result == BAD_PARAMETER)
	cout << "connect result BAD_PARAMETER. " << endl;
      else if (result == UNSUPPORTED)
	cout << "connect result UNSUPPORTED. " << endl;
      else if (result == OUT_OF_RESOURCES)
	cout << "connect result OUT_OF_RESOURCES. " << endl;
      else if (result == PRECONDITION_NOT_MET)
	cout << "connect result PRECONDITION_NOT_MET. " << endl;
    }
    
    
    /*!
     * @brief connect()Υƥ
     *   ̤ƥ
     */
    void test_connect() {}
    
    
    /*!
     * @brief disconnect()Υƥ
     *   ̤ƥ
     */
    void test_disconnect() {}
    
    
    /*!
     * @brief disconnect_all()Υƥ
     *   ̤ƥ
     */
    void test_disconnect_all() {}
    
    
    /*!
     * @brief setProfile()Υƥ
     *   test_get_port_profile() ˤƥƥȺѤ
     */
    void test_setProfile() {}
    
    
    /*!
     * @brief getProfile()Υƥ
     *    PortProfilesetUp()ϿѤߤǤ롣
     *   (1) getProfile()ˤPortProfile
     *   (2) åȤPortProfileȼȤǤӡ
     */
    /*
    void test_getProfile() {
      PortProfile getProf;
      
      // (1) getProfile()ˤPortProfile
      // ### getProf = m_ppb->getProfile();
      
      string setstr, getstr;
      
      // (2) åȤPortProfileȼȤǤӡ
      // check PortProfile.name
      getstr = getProf.name;
      setstr = "inport0";
      CPPUNIT_ASSERT(getstr == setstr);
      
      // check PortProfile.interfaces
      getstr = getProf.interfaces[0].instance_name;
      setstr = "PortInterfaceProfile-instance_name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getProf.interfaces[0].type_name;
      setstr = "PortInterfaceProfile-type_name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      CPPUNIT_ASSERT(getProf.interfaces[0].polarity == REQUIRED);
      
      // check PortProfile.connector_profiles
      getstr = getProf.connector_profiles[0].name;
      setstr = "ConnectorProfile-name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getProf.connector_profiles[0].connector_id;
      setstr = "connect_id0";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getProf.connector_profiles[0].properties[0].name;
      setstr = "ConnectorProfile-properties0-name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      CORBA::Float retval;
      getProf.connector_profiles[0].properties[0].value >>= retval;
      CPPUNIT_ASSERT(retval == m_cnctProfVal);
      
      // check PortProfile.properties
      getstr = getProf.properties[0].name;
      setstr = "PortProfile-properties0-name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getProf.properties[0].value >>= retval;
      CPPUNIT_ASSERT(retval == m_portProfVal);
    }
    */
    
    /*!
     * @brief setName()Υƥ
     *    PortProfilesetUp()ˤϿѤߤǤ롣
     *   (1) setName()ˤPortProfile.name򥻥åȡ
     *   (2) getProfile()ˤPortProfile
     *   (3) (1)ǥåȤname(2)ǼPortProfile.nameӡ
     */
    /*
    void test_setName() {
      // (1) setName()ˤPortProfile.name򥻥åȡ
      m_ppb->setName("inport0-changed");
      
      PortProfile getProf;
      
      // (2) getProfile()ˤPortProfile
      getProf = m_ppb->getProfile();
      
      string setstr, getstr;
      
      // (3) (1)ǥåȤname(2)ǼPortProfile.nameӡ
      getstr = getProf.name;
      setstr = "inport0-changed";
      CPPUNIT_ASSERT(getstr == setstr);
    }
    */
    
    /*!
     * @brief  getName()Υƥ
     *    PortProfilesetUp()ˤϿѤߤǤ롣
     *   (1) geName()ˤPortProfile.name
     *   (2) åȺѤߤPortProfile.name(1)Ǽnameӡ
     */
    /*
    void test_getName() {
      // (1) geName()ˤPortProfile.name
      const char* retval(m_ppb->getName());
      
      string setname, getname;
      
      // (2) åȺѤߤPortProfile.name(1)Ǽnameӡ
      setname = "inport0";
      getname = retval;
      CPPUNIT_ASSERT(getname == setname);
    }
    */
    
    /*!
     * @brief setInterfaceProfiles()Υƥ
     *   (1) PortInterfaceProfile֥ǤΥåȡ
     *   (2) (1)ǥåȤPortInterfaceProfilePortInterfaceProfileList
     *       åȡ
     *   (3setInterfaceProfiles()ˤPortInterfaceProfileList򥻥åȡ
     *   (4) getInterfaceProfiles()ˤPortInterfaceProfileList
     *   (5) (3)ǥåȤPortInterfaceProfileListǤȡ
     *       (4)ǼȤӡ
     *   (6) addInterfaceProfile()ˤPortInterfaceProfileɲá
     *   (7) getInterfaceProfiles()ˤPortInterfaceProfileList
     *   (8) (6)ɲäPortInterfaceProfileListǤȡ
     *       (7)ǼȤӡ
     *   (9) getInterfaceProfile(name)ˤPortInterfaceProfile
     *   (10) (6)ɲäPortInterfaceProfileListǤȡ
     *        (9)ǼȤӡ
     */
    /*
    void test_setInterfaceProfiles() {
      // (1) PortInterfaceProfile֥ǤΥåȡ
      PortInterfaceProfile pIProf;
      pIProf.instance_name = "PortInterfaceProfile-instance_name-changed";
      pIProf.type_name = "PortInterfaceProfile-type_name-changed";
      pIProf.polarity = REQUIRED;
      
      
      // (2) (1)ǥåȤPortInterfaceProfilePortInterfaceProfileList
      // ˥åȡ
      PortInterfaceProfileList pIFProfList;
      pIFProfList.length(1);
      pIFProfList[0] = pIProf;
      
      
      // (3setInterfaceProfiles()ˤPortInterfaceProfileList򥻥åȡ
      m_ppb->setInterfaceProfiles(pIFProfList);
      
      
      // (4) getInterfaceProfiles()ˤPortInterfaceProfileList
      PortInterfaceProfileList getList;
      getList = m_ppb->getInterfaceProfiles();
      
      
      // (5) (3)ǥåȤPortInterfaceProfileListǤȡ
      // (4)ǼȤӡ
      string getstr, setstr;
      getstr = getList[0].instance_name;
      setstr = "PortInterfaceProfile-instance_name-changed";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getList[0].type_name;
      setstr = "PortInterfaceProfile-type_name-changed";
      CPPUNIT_ASSERT(getstr == setstr);
      
      CPPUNIT_ASSERT(getList[0].polarity == REQUIRED);
      
      
      // (6) addInterfaceProfile()ˤPortInterfaceProfileɲá
      pIProf.instance_name = "PortInterfaceProfile-instance_name-add";
      pIProf.type_name = "PortInterfaceProfile-type_name-add";
      pIProf.polarity = PROVIDED;
      m_ppb->addInterfaceProfile(pIProf);
      
      
      // (7) getInterfaceProfiles()ˤPortInterfaceProfileList
      getList = m_ppb->getInterfaceProfiles();
      
      
      // (8) (6)ɲäPortInterfaceProfileListǤȡ
      // (7)ǼȤӡ
      getstr = getList[1].instance_name;
      setstr = "PortInterfaceProfile-instance_name-add";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getList[1].type_name;
      setstr = "PortInterfaceProfile-type_name-add";
      CPPUNIT_ASSERT(getstr == setstr);
      
      CPPUNIT_ASSERT(getList[1].polarity == PROVIDED);
      
      
      // (9) getInterfaceProfile(name)ˤPortInterfaceProfile
      PortInterfaceProfile getProf;
      getProf = m_ppb->getInterfaceProfile("PortInterfaceProfile-instance_name-add");
      
      // (10) (6)ɲäPortInterfaceProfileListǤȡ(9)ǼȤӡ
      getstr = getProf.instance_name;
      setstr = "PortInterfaceProfile-instance_name-add";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getProf.type_name;
      setstr = "PortInterfaceProfile-type_name-add";
      CPPUNIT_ASSERT(getstr == setstr);
      
      CPPUNIT_ASSERT(getProf.polarity == PROVIDED);
    }
    */
    /*!
     * @brief addInterfaceProfiles()Υƥ
     *   test_setInterfaceProfiles()ˤƥƥȺѤߡ
     */
    void test_addInterfaceProfiles() {}
    
    
    /*!
     * @brief getInterfaceProfiles()Υƥ
     *   test_setInterfaceProfiles()ˤƥƥȺѤߡ
     */
    void test_getInterfaceProfiles() {}
    
    
    /*!
     * @brief getInterfaceProfile()Υƥ
     *   test_setInterfaceProfiles()ˤƥƥȺѤߡ
     */
    void test_getInterfaceProfile() {}
    
    
    
    /*!
     * @brief  setPortRef()Υƥ
     *    PortProfilesetUp()ˤϿѤߡ
     *   (1) setPortRef()ˤPortBase֥ȤλȤ򥻥åȡ
     *   (2) getPortRef()ˤPort󥿥եΥ֥ȻȤ
     *   (3) (2)Ǽ֥ȻȤѤPort󥿥ե
     *       ڥ졼ƤӽФƥȡ
     *   (4) (3)Υڥ졼ƤӽФˤƼPortProfileǤ
     *       setUp()ǥåȤȤӡ
     */
    void test_setPortRef() {
      Port_var port = m_ppb->_this();
      m_ppb->_remove_ref();
      
      // (1) setPortRef()ˤPortBase֥ȤλȤ򥻥åȡ
      m_ppb->setPortRef(port._retn());
      
      
      // (2) getPortRef()ˤPort󥿥եΥ֥ȻȤ
      Port_ptr getP;
      PortProfile* pProf;
      getP = m_ppb->getPortRef();
      
      
      // (3) (2)Ǽ֥ȻȤѤPort󥿥եΥڥ졼ƤӽФƥȡ
      pProf = getP->get_port_profile();
      
      
      // (4) (3)Υڥ졼ƤӽФˤƼPortProfileǤsetUp()
      //     åȤȤӡ
      string setstr, getstr;
      getstr = pProf->name;
      setstr = "inport0";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = pProf->interfaces[0].instance_name;
      setstr = "PortInterfaceProfile-instance_name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = pProf->interfaces[0].type_name;
      setstr = "PortInterfaceProfile-type_name";
      CPPUNIT_ASSERT(getstr == setstr);
      
      CORBA::Float getval;
      pProf->properties[0].value >>= getval;
      CPPUNIT_ASSERT(m_portProfVal == getval);
      
      getstr = pProf->properties[0].name;
      setstr = "PortProfile-properties0-name";
      CPPUNIT_ASSERT(getstr == setstr);
    }
    
    
    /*!
     * @brief getPortRefΥƥ
     *   test_setPortRef()ˤƥƥȺѤߡ
     */
    void test_getPortRef() {}
    
    
    /*!
     * @brief addConnectorProfile()Υƥ
     *    PortProfilesetUp()ϿѤߡ
     *   (1) ConnectorProfile֥ǤΥå
     *   (2) (1)ǥåȤConnectorProfileaddConnectorProfile()ˤɲá
     *   (3ConnectorProfileListμ
     *   (4) (2)ɲäConnectorProfile(3)ǼȤӡ
     *   (5) eraseConnectorProfile(id)ˤ,ǻꤷid
     *       ConnectorProfileConnectorProfileList롣
     *   (6) (5)ǺConnectorProfileList
     *   (7) (6)ǼConnectorProfileListǤǧ
     */
    /*
    void test_addConnectorProfile() {
      // (1) ConnectorProfile֥ǤΥå
      ConnectorProfile cProf,getProf;
      cProf.name = "ConnectorProfile-name-add";
      cProf.connector_id = "connect_id0-add";
      
      SDOPackage::NameValue nv;
      NVList nvlist;
      CORBA::Double db, retdb;
      nv.name = "ConnectorProfile-properties0-name-add";
      db = 9999999.999;
      nv.value <<= db;
      nvlist.length(1);
      nvlist[0] = nv;
      cProf.properties = nvlist;
      
      
      // (2) (1)ǥåȤConnectorProfileaddConnectorProfile()ˤɲá
      m_ppb->addConnectorProfile(cProf);
      
      
      // (3ConnectorProfileListμ
      ConnectorProfileList cnctPList;
      cnctPList = m_ppb->getConnectorProfileList();
      
      
      // (4) (2)ɲäConnectorProfile(3)ǼȤӡ
      string setstr, getstr;
      getstr = cnctPList[1].name;
      setstr = cProf.name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = cnctPList[1].connector_id;
      setstr = cProf.connector_id;
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = cnctPList[1].properties[0].name;
      setstr = nv.name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      cnctPList[1].properties[0].value >>= retdb;
      CPPUNIT_ASSERT(retdb == db);
      
      
      // (5) eraseConnectorProfile(id)ˤ,ǻꤷid
      //     ConnectorProfileConnectorProfileList롣
      // test eraseConnectorProfile()
      // Failure case: ϿƤʤIDeraseConnectorProfile()
      // Ƥ֤ȥܡȤ롣
      //    m_ppb->eraseConnectorProfile("hoge");
      m_ppb->eraseConnectorProfile("connect_id0");
      
      
      // (6) (5)ǺConnectorProfileList
      getProf = m_ppb->getConnectorProfile("connect_id0-add");
      
      
      // (7) (6)ǼConnectorProfileListǤǧ
      getstr = getProf.name;
      setstr = getProf.name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getProf.connector_id;
      setstr = getProf.connector_id;
      CPPUNIT_ASSERT(getstr == setstr);
      
      getstr = getProf.properties[0].name;
      setstr = nv.name;
      CPPUNIT_ASSERT(getstr == setstr);
      
      getProf.properties[0].value >>= retdb;
      CPPUNIT_ASSERT(retdb == db);
    }
    */
    
    /*!
     * @brief  eraseConnectorProfile()Υƥ
     *   test_addConnectorProfileList()ˤƥƥȺѤߡ
     */
    void test_eraseConnectorProfile() {}
    
    
    /*!
     * @brief getConnectorProfileList()Υƥ
     *   test_addConnectorProfileList()ˤƥƥȺѤߡ
     */
    void test_getConnectorProfileList() {}
    
    
    /*!
     * @brief getConnectorProfile()Υƥ
     *   test_addConnectorProfileList()ˤƥƥȺѤߡ
     */
    void test_getConnectorProfile() {}
    
    
    /*!
     * @brief setOwner()
     *   RTObject󥿥եμλƤʤ
     */
    void test_setOwner() {
      /*
	RTObject_impl* rtobj;
	
	rtobj = new RTObject_impl(m_orb, m_poa);
	
	RTObject_var p_rtobj = rtobj->_this();
	rtobj->_remove_ref();
	
	// check setOwner()
	m_ppb->setOwner(p_rtobj._retn());
      */
      
    }
    
    /*!
     * @brief getOwner()Υƥ
     */
    void test_getOwner() {}
    
    
    /*!
     * @brief setProperties()Υƥ
     *    PortProfilesetUp()ϿѤߤǤ롣
     *   (1) getProperties()ˤPortProfile.property
     *   (2) (1)ǼpropertyǤsetUp()ǥåȤȤӡ
     *   (3) setProperties()ˤPortProfile.property򥻥å
     *   (4) getProperties()ˤPortProfile.property
     *   (5) (3)ǥåȤPortProfile.property(4)ǼȤӡ
     */
	/*
    void test_setProperties() {
      NVList setlist, getlist;
      SDOPackage::NameValue nv;
      
      // (1) getProperties()ˤPortProfile.property
      getlist = m_ppb->getProperties();
      
      
      // (2) (1)ǼpropertyǤsetUp()ǥåȤȤӡ
      string getstr, setstr;
      CORBA::Float setval, retval;
      getlist[0].value >>= retval;
      
      getstr = getlist[0].name;
      setstr = m_nv.name;
      CPPUNIT_ASSERT(getstr == setstr);
      CPPUNIT_ASSERT(retval == m_portProfVal);
      
      
      // (3) setProperties()ˤPortProfile.property򥻥å
      nv.name = "setProperties test";
      setval = 999.9;
      nv.value <<= setval;
      
      setlist.length(1);
      setlist[0] = nv;
      m_ppb->setProperties(setlist);
      
      
      // (4) getProperties()ˤPortProfile.property
      getlist = m_ppb->getProperties();
      
      
      // (5) (3)ǥåȤPortProfile.property(4)ǼȤӡ
      getlist[0].value >>= retval;
      
      getstr = getlist[0].name;
      setstr = nv.name;
      CPPUNIT_ASSERT(getstr == setstr);
      CPPUNIT_ASSERT(retval == setval);
      
    }
    */
    
    /*!
     * @brief getProperties()Υƥ
     *   test_setPropertiesˤƥƥȺѤߡ
     */
	/*
    void test_getProperties() {}
    */
    
    /*!
     * @brief  getUUID()Υƥ
     *   (1) getUUID()ˤUUIDμ
     *   (2) UUIDɸϤ˽ϡ
     */
    void test_getUUID() {
      // (1) getUUID()ˤUUIDμ
      string getuuid = m_ppb->getUUID();
      
      // (2) UUIDɸϤ˽ϡ
      cout << endl << "uuid: " << getuuid << endl;
    }
    
  };
}; // namespace PortBase

/*
 * Register test suite
 */
CPPUNIT_TEST_SUITE_REGISTRATION(PortBase::PortBaseTests);

#ifdef LOCAL_MAIN
int main(int argc, char* argv[])
{
  CppUnit::TextUi::TestRunner runner;
  runner.addTest(CppUnit::TestFactoryRegistry::getRegistry().makeTest());
  CppUnit::Outputter* outputter = 
    new CppUnit::TextOutputter(&runner.result(), std::cout);
  runner.setOutputter(outputter);
  bool retcode = runner.run();
  return !retcode;
}
#endif // MAIN
#endif // PortBase_cpp
